"""
This type stub file was generated by pyright.
"""

"""
Reversed Operations not available in the stdlib operator module.
Defining these instead of using lambdas allows us to reference them by name.
"""
def radd(left, right):
    ...

def rsub(left, right):
    ...

def rmul(left, right):
    ...

def rdiv(left, right):
    ...

def rtruediv(left, right):
    ...

def rfloordiv(left, right):
    ...

def rmod(left, right):
    ...

def rdivmod(left, right):
    ...

def rpow(left, right):
    ...

def rand_(left, right): # -> Any:
    ...

def ror_(left, right): # -> Any:
    ...

def rxor(left, right): # -> Any:
    ...

