"""
This type stub file was generated by pyright.
"""

from typing import NamedTuple, TYPE_CHECKING
from pandas._typing import ArrayLike
from pandas._libs.internals import BlockPlacement
from pandas.core.internals.blocks import Block
from pandas.core.internals.managers import BlockManager

if TYPE_CHECKING:
    ...
class BlockPairInfo(NamedTuple):
    lvals: ArrayLike
    rvals: ArrayLike
    locs: BlockPlacement
    left_ea: bool
    right_ea: bool
    rblk: Block
    ...


def operate_blockwise(left: BlockManager, right: BlockManager, array_op) -> BlockManager:
    ...

def blockwise_all(left: BlockManager, right: BlockManager, op) -> bool:
    """
    Blockwise `all` reduction.
    """
    ...

