"""
This type stub file was generated by pyright.
"""

import numpy as np

"""
transforms.py is for shape-preserving functions.
"""
def shift(values: np.ndarray, periods: int, axis: int, fill_value) -> np.ndarray:
    ...

