"""
This type stub file was generated by pyright.
"""

from pandas._libs.lib import no_default
from pandas.core.dtypes.base import ExtensionDtype, register_extension_dtype
from pandas.core.accessor import register_dataframe_accessor, register_index_accessor, register_series_accessor
from pandas.core.algorithms import take
from pandas.core.arrays import ExtensionArray, ExtensionScalarOpsMixin

"""
Public API for extending pandas objects.
"""
__all__ = ["no_default", "ExtensionDtype", "register_extension_dtype", "register_dataframe_accessor", "register_index_accessor", "register_series_accessor", "take", "ExtensionArray", "ExtensionScalarOpsMixin"]
