"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._typing import npt

def unstack(values: np.ndarray, mask: np.ndarray, stride: int, length: int, width: int, new_values: np.ndarray, new_mask: np.ndarray) -> None:
    ...

def explode(values: npt.NDArray[np.object_]) -> tuple[npt.NDArray[np.object_], npt.NDArray[np.int64]]:
    ...

