"""
This type stub file was generated by pyright.
"""

"""
This module is imported from the pandas package __init__.py file
in order to ensure that the core.config options registered here will
be available as soon as the user loads the package. if register_option
is invoked inside specific modules, they will not be registered until that
module is imported, which may or may not be a problem.

If you need to make sure options are available even before a certain
module is imported, register them here rather than in the module.

"""
use_bottleneck_doc = ...
def use_bottleneck_cb(key) -> None:
    ...

use_numexpr_doc = ...
def use_numexpr_cb(key) -> None:
    ...

use_numba_doc = ...
def use_numba_cb(key) -> None:
    ...

pc_precision_doc = ...
pc_colspace_doc = ...
pc_max_rows_doc = ...
pc_min_rows_doc = ...
pc_max_cols_doc = ...
pc_max_categories_doc = ...
pc_max_info_cols_doc = ...
pc_nb_repr_h_doc = ...
pc_pprint_nest_depth = ...
pc_multi_sparse_doc = ...
float_format_doc = ...
max_colwidth_doc = ...
colheader_justify_doc = ...
pc_expand_repr_doc = ...
pc_show_dimensions_doc = ...
pc_east_asian_width_doc = ...
pc_ambiguous_as_wide_doc = ...
pc_latex_repr_doc = ...
pc_table_schema_doc = ...
pc_html_border_doc = ...
pc_html_use_mathjax_doc = ...
pc_max_dir_items = ...
pc_width_doc = ...
pc_chop_threshold_doc = ...
pc_max_seq_items = ...
pc_max_info_rows_doc = ...
pc_large_repr_doc = ...
pc_memory_usage_doc = ...
pc_latex_escape = ...
pc_latex_longtable = ...
pc_latex_multicolumn = ...
pc_latex_multicolumn_format = ...
pc_latex_multirow = ...
def table_schema_cb(key) -> None:
    ...

def is_terminal() -> bool:
    """
    Detect if Python is running in a terminal.

    Returns True if Python is running in a terminal or False if not.
    """
    ...

tc_sim_interactive_doc = ...
use_inf_as_null_doc = ...
use_inf_as_na_doc = ...
def use_inf_as_na_cb(key) -> None:
    ...

data_manager_doc = ...
copy_on_write_doc = ...
chained_assignment = ...
string_storage_doc = ...
reader_engine_doc = ...
_xls_options = ...
_xlsm_options = ...
_xlsx_options = ...
_ods_options = ...
_xlsb_options = ...
writer_engine_doc = ...
_xls_options = ...
_xlsm_options = ...
_xlsx_options = ...
_ods_options = ...
parquet_engine_doc = ...
sql_engine_doc = ...
plotting_backend_doc = ...
def register_plotting_backend_cb(key) -> None:
    ...

register_converter_doc = ...
def register_converter_cb(key) -> None:
    ...

styler_sparse_index_doc = ...
styler_sparse_columns_doc = ...
styler_render_repr = ...
styler_max_elements = ...
styler_max_rows = ...
styler_max_columns = ...
styler_precision = ...
styler_decimal = ...
styler_thousands = ...
styler_na_rep = ...
styler_escape = ...
styler_formatter = ...
styler_multirow_align = ...
styler_multicol_align = ...
styler_hrules = ...
styler_environment = ...
styler_encoding = ...
styler_mathjax = ...
