"""
This type stub file was generated by pyright.
"""

from pandas.io.sas.sas7bdat import SAS7BDATReader

class Parser:
    def __init__(self, parser: SAS7BDATReader) -> None:
        ...
    
    def read(self, nrows: int) -> None:
        ...
    


