"""
This type stub file was generated by pyright.
"""

from pandas.io.parsers.readers import TextFileReader, TextParser, read_csv, read_fwf, read_table

__all__ = ["TextFileReader", "TextParser", "read_csv", "read_fwf", "read_table"]
