"""
This type stub file was generated by pyright.
"""

from enum import Enum
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    ...
class ParserBase:
    class BadLineHandleMethod(Enum):
        ERROR = ...
        WARN = ...
        SKIP = ...
    
    
    _implicit_index: bool = ...
    _first_chunk: bool
    def __init__(self, kwds) -> None:
        ...
    
    def close(self) -> None:
        ...
    


parser_defaults = ...
def is_index_col(col) -> bool:
    ...

