"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING

""" define generic base classes for pandas objects """
if TYPE_CHECKING:
    ...
def create_pandas_abc_type(name, attr, comp): # -> Type[__class_ABCBase]:
    ...

ABCInt64Index = ...
ABCUInt64Index = ...
ABCRangeIndex = ...
ABCFloat64Index = ...
ABCMultiIndex = ...
ABCDatetimeIndex = ...
ABCTimedeltaIndex = ...
ABCPeriodIndex = ...
ABCCategoricalIndex = ...
ABCIntervalIndex = ...
ABCIndex = ...
ABCNDFrame = ...
ABCSeries = ...
ABCDataFrame = ...
ABCCategorical = ...
ABCDatetimeArray = ...
ABCTimedeltaArray = ...
ABCPeriodArray = ...
ABCExtensionArray = ...
ABCPandasArray = ...
