"""
This type stub file was generated by pyright.
"""

import pyarrow
from pandas._typing import IntervalClosedType

class ArrowPeriodType(pyarrow.ExtensionType):
    def __init__(self, freq) -> None:
        ...
    
    @property
    def freq(self): # -> Unknown:
        ...
    
    def __arrow_ext_serialize__(self) -> bytes:
        ...
    
    @classmethod
    def __arrow_ext_deserialize__(cls, storage_type, serialized) -> ArrowPeriodType:
        ...
    
    def __eq__(self, other) -> bool:
        ...
    
    def __hash__(self) -> int:
        ...
    
    def to_pandas_dtype(self): # -> PeriodDtype | PandasExtensionDtype:
        ...
    


_period_type = ...
class ArrowIntervalType(pyarrow.ExtensionType):
    def __init__(self, subtype, closed: IntervalClosedType) -> None:
        ...
    
    @property
    def subtype(self):
        ...
    
    @property
    def closed(self) -> IntervalClosedType:
        ...
    
    def __arrow_ext_serialize__(self) -> bytes:
        ...
    
    @classmethod
    def __arrow_ext_deserialize__(cls, storage_type, serialized) -> ArrowIntervalType:
        ...
    
    def __eq__(self, other) -> bool:
        ...
    
    def __hash__(self) -> int:
        ...
    
    def to_pandas_dtype(self): # -> IntervalDtype | PandasExtensionDtype:
        ...
    


_interval_type = ...
