"""
This type stub file was generated by pyright.
"""

from pandas.core.groupby.generic import DataFrameGroupBy, NamedAgg, SeriesGroupBy
from pandas.core.groupby.groupby import GroupBy
from pandas.core.groupby.grouper import Grouper

__all__ = ["DataFrameGroupBy", "NamedAgg", "SeriesGroupBy", "GroupBy", "Grouper"]
