"""
This type stub file was generated by pyright.
"""

ne = ...
NUMEXPR_INSTALLED = ...
if NUMEXPR_INSTALLED:
    NUMEXPR_VERSION = ...
else:
    NUMEXPR_VERSION = ...
__all__ = ["NUMEXPR_INSTALLED", "NUMEXPR_VERSION"]
