"""
This type stub file was generated by pyright.
"""

from typing import Any, TYPE_CHECKING
from pandas._typing import FilePath, StorageOptions, WriteExcelBuffer
from pandas.io.excel._base import ExcelWriter

if TYPE_CHECKING:
    ...
class ODSWriter(ExcelWriter):
    _engine = ...
    _supported_extensions = ...
    def __init__(self, path: FilePath | WriteExcelBuffer | ExcelWriter, engine: str | None = ..., date_format: str | None = ..., datetime_format=..., mode: str = ..., storage_options: StorageOptions = ..., if_sheet_exists: str | None = ..., engine_kwargs: dict[str, Any] | None = ..., **kwargs) -> None:
        ...
    
    @property
    def book(self):
        """
        Book instance of class odf.opendocument.OpenDocumentSpreadsheet.

        This attribute can be used to access engine-specific features.
        """
        ...
    
    @property
    def sheets(self) -> dict[str, Any]:
        """Mapping of sheet names to sheet objects."""
        ...
    


