"""
This type stub file was generated by pyright.
"""

def generate_online_numba_ewma_func(nopython: bool, nogil: bool, parallel: bool): # -> (values: ndarray[Unknown, Unknown], deltas: ndarray[Unknown, Unknown], minimum_periods: int, old_wt_factor: float, new_wt: float, old_wt: ndarray[Unknown, Unknown], adjust: bool, ignore_na: bool) -> tuple[NDArray[float64], ndarray[Unknown, Unknown]]:
    """
    Generate a numba jitted groupby ewma function specified by values
    from engine_kwargs.

    Parameters
    ----------
    nopython : bool
        nopython to be passed into numba.jit
    nogil : bool
        nogil to be passed into numba.jit
    parallel : bool
        parallel to be passed into numba.jit

    Returns
    -------
    Numba function
    """
    ...

class EWMMeanState:
    def __init__(self, com, adjust, ignore_na, axis, shape) -> None:
        ...
    
    def run_ewm(self, weighted_avg, deltas, min_periods, ewm_func):
        ...
    
    def reset(self) -> None:
        ...
    


