"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas.util.version import Version

""" support numpy compatibility across versions """
_np_version = ...
_nlv = ...
np_version_under1p21 = ...
np_version_under1p22 = ...
np_version_gte1p22 = ...
is_numpy_dev = ...
_min_numpy_ver = ...
if is_numpy_dev or notnp_version_under1p22:
    np_percentile_argname = ...
else:
    np_percentile_argname = ...
if _nlv < Version(_min_numpy_ver):
    ...
__all__ = ["np", "_np_version", "is_numpy_dev"]
