"""
This type stub file was generated by pyright.
"""

from typing import Final

magic: Final = ...
align_1_checker_value: Final = ...
align_1_offset: Final = ...
align_1_length: Final = ...
align_1_value: Final = ...
u64_byte_checker_value: Final = ...
align_2_offset: Final = ...
align_2_length: Final = ...
align_2_value: Final = ...
endianness_offset: Final = ...
endianness_length: Final = ...
platform_offset: Final = ...
platform_length: Final = ...
encoding_offset: Final = ...
encoding_length: Final = ...
dataset_offset: Final = ...
dataset_length: Final = ...
file_type_offset: Final = ...
file_type_length: Final = ...
date_created_offset: Final = ...
date_created_length: Final = ...
date_modified_offset: Final = ...
date_modified_length: Final = ...
header_size_offset: Final = ...
header_size_length: Final = ...
page_size_offset: Final = ...
page_size_length: Final = ...
page_count_offset: Final = ...
page_count_length: Final = ...
sas_release_offset: Final = ...
sas_release_length: Final = ...
sas_server_type_offset: Final = ...
sas_server_type_length: Final = ...
os_version_number_offset: Final = ...
os_version_number_length: Final = ...
os_maker_offset: Final = ...
os_maker_length: Final = ...
os_name_offset: Final = ...
os_name_length: Final = ...
page_bit_offset_x86: Final = ...
page_bit_offset_x64: Final = ...
subheader_pointer_length_x86: Final = ...
subheader_pointer_length_x64: Final = ...
page_type_offset: Final = ...
page_type_length: Final = ...
block_count_offset: Final = ...
block_count_length: Final = ...
subheader_count_offset: Final = ...
subheader_count_length: Final = ...
page_type_mask: Final = ...
page_type_mask2: Final = ...
page_meta_type: Final = ...
page_data_type: Final = ...
page_mix_type: Final = ...
page_amd_type: Final = ...
page_meta2_type: Final = ...
page_comp_type: Final = ...
page_meta_types: Final = ...
subheader_pointers_offset: Final = ...
truncated_subheader_id: Final = ...
compressed_subheader_id: Final = ...
compressed_subheader_type: Final = ...
text_block_size_length: Final = ...
row_length_offset_multiplier: Final = ...
row_count_offset_multiplier: Final = ...
col_count_p1_multiplier: Final = ...
col_count_p2_multiplier: Final = ...
row_count_on_mix_page_offset_multiplier: Final = ...
column_name_pointer_length: Final = ...
column_name_text_subheader_offset: Final = ...
column_name_text_subheader_length: Final = ...
column_name_offset_offset: Final = ...
column_name_offset_length: Final = ...
column_name_length_offset: Final = ...
column_name_length_length: Final = ...
column_data_offset_offset: Final = ...
column_data_length_offset: Final = ...
column_data_length_length: Final = ...
column_type_offset: Final = ...
column_type_length: Final = ...
column_format_text_subheader_index_offset: Final = ...
column_format_text_subheader_index_length: Final = ...
column_format_offset_offset: Final = ...
column_format_offset_length: Final = ...
column_format_length_offset: Final = ...
column_format_length_length: Final = ...
column_label_text_subheader_index_offset: Final = ...
column_label_text_subheader_index_length: Final = ...
column_label_offset_offset: Final = ...
column_label_offset_length: Final = ...
column_label_length_offset: Final = ...
column_label_length_length: Final = ...
rle_compression: Final = ...
rdc_compression: Final = ...
compression_literals: Final = ...
encoding_names: Final = ...
class SASIndex:
    row_size_index: Final = ...
    column_size_index: Final = ...
    subheader_counts_index: Final = ...
    column_text_index: Final = ...
    column_name_index: Final = ...
    column_attributes_index: Final = ...
    format_and_label_index: Final = ...
    column_list_index: Final = ...
    data_subheader_index: Final = ...


subheader_signature_to_index: Final = ...
sas_date_formats: Final = ...
sas_datetime_formats: Final = ...
