"""
This type stub file was generated by pyright.
"""

from typing import Sequence, overload
from pandas._typing import AnyArrayLike, DataFrame, Index, Series

cache_readonly = property
class AxisProperty:
    axis: int
    def __init__(self, axis: int = ..., doc: str = ...) -> None:
        ...
    
    @overload
    def __get__(self, obj: DataFrame | Series, type) -> Index:
        ...
    
    @overload
    def __get__(self, obj: None, type) -> AxisProperty:
        ...
    
    def __set__(self, obj: DataFrame | Series, value: AnyArrayLike | Sequence) -> None:
        ...
    


