"""
This type stub file was generated by pyright.
"""

import dataclasses
from typing import Hashable, Literal

"""
Provide basic components for groupby. These definitions
hold the allowlist of methods that are exposed on the
SeriesGroupBy and the DataFrameGroupBy objects.
"""
@dataclasses.dataclass(order=True, frozen=True)
class OutputKey:
    label: Hashable
    position: int
    ...


plotting_methods = ...
common_apply_allowlist = ...
series_apply_allowlist: frozenset[str] = ...
dataframe_apply_allowlist: frozenset[str] = ...
cythonized_kernels = ...
reduction_kernels = ...
def maybe_normalize_deprecated_kernels(kernel) -> Literal["bfill", "ffill"]:
    ...

transformation_kernels = ...
groupby_other_methods = ...
transform_kernel_allowlist = ...
