"""
This type stub file was generated by pyright.
"""

import numba
import numpy as np

"""
Numba 1D min/max kernels that can be shared by
* Dataframe / Series
* groupby
* rolling / expanding

Mirrors pandas/_libs/window/aggregation.pyx
"""
@numba.jit(nopython=True, nogil=True, parallel=False)
def sliding_min_max(values: np.ndarray, start: np.ndarray, end: np.ndarray, min_periods: int, is_max: bool) -> np.ndarray:
    ...

