"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._typing import npt

"""
masked_reductions.py is for reduction algorithms using a mask-based approach
for missing values.
"""
def sum(values: np.ndarray, mask: npt.NDArray[np.bool_], *, skipna: bool = ..., min_count: int = ..., axis: int | None = ...): # -> NAType:
    ...

def prod(values: np.ndarray, mask: npt.NDArray[np.bool_], *, skipna: bool = ..., min_count: int = ..., axis: int | None = ...): # -> NAType:
    ...

def min(values: np.ndarray, mask: npt.NDArray[np.bool_], *, skipna: bool = ..., axis: int | None = ...): # -> NAType:
    ...

def max(values: np.ndarray, mask: npt.NDArray[np.bool_], *, skipna: bool = ..., axis: int | None = ...): # -> NAType:
    ...

def mean(values: np.ndarray, mask: npt.NDArray[np.bool_], skipna: bool = ...): # -> NAType:
    ...

