"""
This type stub file was generated by pyright.
"""

"""
Data IO api
"""
__all__ = ["ExcelFile", "ExcelWriter", "HDFStore", "read_clipboard", "read_csv", "read_excel", "read_feather", "read_fwf", "read_gbq", "read_hdf", "read_html", "read_json", "read_orc", "read_parquet", "read_pickle", "read_sas", "read_spss", "read_sql", "read_sql_query", "read_sql_table", "read_stata", "read_table", "read_xml", "to_pickle"]
