"""
This type stub file was generated by pyright.
"""

import numpy as np

"""
Templates for invalid operations.
"""
def invalid_comparison(left, right, op) -> np.ndarray:
    """
    If a comparison has mismatched types and is not necessarily meaningful,
    follow python3 conventions by:

        - returning all-False for equality
        - returning all-True for inequality
        - raising TypeError otherwise

    Parameters
    ----------
    left : array-like
    right : scalar, array-like
    op : operator.{eq, ne, lt, le, gt}

    Raises
    ------
    TypeError : on inequality comparisons
    """
    ...

def make_invalid_op(name: str): # -> (self: Unknown, other: Unknown | None = None) -> NoReturn:
    """
    Return a binary method that always raises a TypeError.

    Parameters
    ----------
    name : str

    Returns
    -------
    invalid_op : function
    """
    ...

