"""
This type stub file was generated by pyright.
"""

import numpy as np
from datetime import datetime
from pandas._libs.tslibs.offsets import BaseOffset
from pandas._typing import npt

class DateParseError(ValueError):
    ...


def parse_datetime_string(date_string: str, dayfirst: bool = ..., yearfirst: bool = ..., **kwargs) -> datetime:
    ...

def parse_time_string(arg: str, freq: BaseOffset | str | None = ..., dayfirst: bool | None = ..., yearfirst: bool | None = ...) -> tuple[datetime, str]:
    ...

def quarter_to_myear(year: int, quarter: int, freq: str) -> tuple[int, int]:
    ...

def try_parse_dates(values: npt.NDArray[np.object_], parser=..., dayfirst: bool = ..., default: datetime | None = ...) -> npt.NDArray[np.object_]:
    ...

def try_parse_date_and_time(dates: npt.NDArray[np.object_], times: npt.NDArray[np.object_], date_parser=..., time_parser=..., dayfirst: bool = ..., default: datetime | None = ...) -> npt.NDArray[np.object_]:
    ...

def try_parse_year_month_day(years: npt.NDArray[np.object_], months: npt.NDArray[np.object_], days: npt.NDArray[np.object_]) -> npt.NDArray[np.object_]:
    ...

def try_parse_datetime_components(years: npt.NDArray[np.object_], months: npt.NDArray[np.object_], days: npt.NDArray[np.object_], hours: npt.NDArray[np.object_], minutes: npt.NDArray[np.object_], seconds: npt.NDArray[np.object_]) -> npt.NDArray[np.object_]:
    ...

def format_is_iso(f: str) -> bool:
    ...

def guess_datetime_format(dt_str, dayfirst: bool | None = ...) -> str | None:
    ...

def concat_date_cols(date_cols: tuple, keep_trivial_numbers: bool = ...) -> npt.NDArray[np.object_]:
    ...

def get_rule_month(source: str) -> str:
    ...

