"""
This type stub file was generated by pyright.
"""

from pandas.core.arrays.sparse.accessor import SparseAccessor, SparseFrameAccessor
from pandas.core.arrays.sparse.array import BlockIndex, IntIndex, SparseArray, make_sparse_index
from pandas.core.arrays.sparse.dtype import SparseDtype

__all__ = ["BlockIndex", "IntIndex", "make_sparse_index", "SparseAccessor", "SparseArray", "SparseDtype", "SparseFrameAccessor"]
