"""
This type stub file was generated by pyright.
"""

from pandas._typing import Scalar, StorageOptions
from pandas.util._decorators import doc
from pandas.core.shared_docs import _shared_docs
from pandas.io.excel._base import BaseExcelReader

class XlrdReader(BaseExcelReader):
    @doc(storage_options=_shared_docs["storage_options"])
    def __init__(self, filepath_or_buffer, storage_options: StorageOptions = ...) -> None:
        """
        Reader using xlrd engine.

        Parameters
        ----------
        filepath_or_buffer : str, path object or Workbook
            Object to be parsed.
        {storage_options}
        """
        ...
    
    def load_workbook(self, filepath_or_buffer):
        ...
    
    @property
    def sheet_names(self):
        ...
    
    def get_sheet_by_name(self, name):
        ...
    
    def get_sheet_by_index(self, index):
        ...
    
    def get_sheet_data(self, sheet, convert_float: bool, file_rows_needed: int | None = ...) -> list[list[Scalar]]:
        ...
    


