"""
This type stub file was generated by pyright.
"""

from pandas.util._decorators import Appender, Substitution, cache_readonly
from pandas.core.util.hashing import hash_array, hash_pandas_object

def __getattr__(name): # -> Module("pandas.util.testing.util.testing"):
    ...

