"""
This type stub file was generated by pyright.
"""

"""Common utility functions for rolling operations"""
def flex_binary_moment(arg1, arg2, f, pairwise=...): # -> DataFrame | None:
    ...

def zsqrt(x): # -> Any:
    ...

def prep_binary(arg1, arg2): # -> tuple[Unknown, Unknown]:
    ...

def maybe_warn_args_and_kwargs(cls, kernel: str, args, kwargs) -> None:
    """
    Warn for deprecation of args and kwargs in rolling/expanding functions.

    Parameters
    ----------
    cls : type
        Class to warn about.
    kernel : str
        Operation name.
    args : tuple or None
        args passed by user. Will be None if and only if kernel does not have args.
    kwargs : dict or None
        kwargs passed by user. Will be None if and only if kernel does not have kwargs.
    """
    ...

