"""
This type stub file was generated by pyright.
"""

from pandas.core.shared_docs import _shared_docs

"""Any shareable docstring components for rolling/expanding/ewm"""
_shared_docs = ...
def create_section_header(header: str) -> str:
    """Create numpydoc section header"""
    ...

template_header = ...
template_returns = ...
template_see_also = ...
kwargs_numeric_only = ...
args_compat = ...
kwargs_compat = ...
kwargs_scipy = ...
window_apply_parameters = ...
numba_notes = ...
def window_agg_numba_parameters(version: str = ...) -> str:
    ...

