"""
This type stub file was generated by pyright.
"""

from pandas import DataFrame

"""
Helpers for sharing tests between DataFrame/Series
"""
def get_dtype(obj): # -> Any:
    ...

def get_obj(df: DataFrame, klass): # -> DataFrame | Series:
    """
    For sharing tests using frame_or_series, either return the DataFrame
    unchanged or return it's first column as a Series.
    """
    ...

