"""
This type stub file was generated by pyright.
"""

from pandas.core.internals.api import make_block
from pandas.core.internals.array_manager import ArrayManager, SingleArrayManager
from pandas.core.internals.base import DataManager, SingleDataManager
from pandas.core.internals.blocks import Block, DatetimeTZBlock, ExtensionBlock, NumericBlock, ObjectBlock
from pandas.core.internals.concat import concatenate_managers
from pandas.core.internals.managers import BlockManager, SingleBlockManager, create_block_manager_from_blocks

__all__ = ["Block", "NumericBlock", "DatetimeTZBlock", "ExtensionBlock", "ObjectBlock", "make_block", "DataManager", "ArrayManager", "BlockManager", "SingleDataManager", "SingleBlockManager", "SingleArrayManager", "concatenate_managers", "create_block_manager_from_blocks"]
def __getattr__(name: str): # -> Type[CategoricalBlock]:
    ...

