"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING
from pandas.core.strings.base import BaseStringArrayMethods

if TYPE_CHECKING:
    ...
class ObjectStringArrayMixin(BaseStringArrayMethods):
    """
    String Methods operating on object-dtype ndarrays.
    """
    _str_na_value = ...
    def __len__(self):
        ...
    


