"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING
from pandas._typing import FilePath, ReadBuffer, Scalar, StorageOptions
from pandas.util._decorators import doc
from pandas.core.shared_docs import _shared_docs
from pandas.io.excel._base import BaseExcelReader
from pandas._libs.tslibs.nattype import NaTType

if TYPE_CHECKING:
    ...
@doc(storage_options=_shared_docs["storage_options"])
class ODFReader(BaseExcelReader):
    """
    Read tables out of OpenDocument formatted files.

    Parameters
    ----------
    filepath_or_buffer : str, path to be parsed or
        an open readable stream.
    {storage_options}
    """
    def __init__(self, filepath_or_buffer: FilePath | ReadBuffer[bytes], storage_options: StorageOptions = ...) -> None:
        ...
    
    def load_workbook(self, filepath_or_buffer: FilePath | ReadBuffer[bytes]):
        ...
    
    @property
    def empty_value(self) -> str:
        """Property for compat with other readers."""
        ...
    
    @property
    def sheet_names(self) -> list[str]:
        """Return a list of sheet names present in the document"""
        ...
    
    def get_sheet_by_index(self, index: int):
        ...
    
    def get_sheet_by_name(self, name: str):
        ...
    
    def get_sheet_data(self, sheet, convert_float: bool, file_rows_needed: int | None = ...) -> list[list[Scalar | NaTType]]:
        """
        Parse an ODF Table into a list of lists
        """
        ...
    


