"""
This type stub file was generated by pyright.
"""

from typing import Callable
from pandas._typing import F

"""
Boilerplate functions used in defining binary operations.
"""
def unpack_zerodim_and_defer(name: str) -> Callable[[F], F]:
    """
    Boilerplate for pandas conventions in arithmetic and comparison methods.

    Parameters
    ----------
    name : str

    Returns
    -------
    decorator
    """
    ...

def get_op_result_name(left, right): # -> None:
    """
    Find the appropriate name to pin to an operation result.  This result
    should always be either an Index or a Series.

    Parameters
    ----------
    left : {Series, Index}
    right : object

    Returns
    -------
    name : object
        Usually a string
    """
    ...

