"""
This type stub file was generated by pyright.
"""

import string
import numpy as np

def randbool(size=..., p: float = ...): # -> bool:
    ...

RANDS_CHARS = np.array(list(string.ascii_letters + string.digits), dtype=(np.str_, 1))
RANDU_CHARS = np.array(list("".join(map(chr, range(1488, 1488 + 26))) + string.digits), dtype=(np.unicode_, 1))
def rands_array(nchars, size, dtype=..., replace=...) -> np.ndarray:
    """
    Generate an array of byte strings.
    """
    ...

def rands(nchars) -> str:
    """
    Generate one random byte string.

    See `rands_array` if you want to create an array of random strings.

    """
    ...

