"""
This type stub file was generated by pyright.
"""

"""
Entrypoint for testing from the top-level namespace.
"""
PKG = ...
def test(extra_args: list[str] | None = ...):
    """
    Run the pandas test suite using pytest.

    By default, runs with the marks --skip-slow, --skip-network, --skip-db

    Parameters
    ----------
    extra_args : list[str], default None
        Extra marks to run the tests.
    """
    ...

__all__ = ["test"]
