"""
This type stub file was generated by pyright.
"""

from pandas.core.arrays.arrow.array import ArrowExtensionArray
from pandas.core.arrays.arrow.dtype import ArrowDtype

__all__ = ["ArrowDtype", "ArrowExtensionArray"]
