"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING
from pandas._typing import ReadBuffer
from pandas.io.parsers.base_parser import ParserBase
from pandas import DataFrame

if TYPE_CHECKING:
    ...
class ArrowParserWrapper(ParserBase):
    """
    Wrapper for the pyarrow engine for read_csv()
    """
    def __init__(self, src: ReadBuffer[bytes], **kwds) -> None:
        ...
    
    def read(self) -> DataFrame:
        """
        Reads the contents of a CSV file into a DataFrame and
        processes it according to the kwargs passed in the
        constructor.

        Returns
        -------
        DataFrame
            The DataFrame created from the CSV file.
        """
        ...
    


