"""
This type stub file was generated by pyright.
"""

def show_versions(as_json: str | bool = ...) -> None:
    """
    Provide useful information, important for bug reports.

    It comprises info about hosting operation system, pandas version,
    and versions of other installed relative packages.

    Parameters
    ----------
    as_json : str or bool, default False
        * If False, outputs info in a human readable form to the console.
        * If str, it will be considered as a path to a file.
          Info will be written to that file in JSON format.
        * If True, outputs info in JSON format to the console.
    """
    ...

def main() -> int:
    ...

if __name__ == "__main__":
    ...
