"""
This type stub file was generated by pyright.
"""

"""
Functions to generate methods and pin them to the appropriate classes.
"""
def add_flex_arithmetic_methods(cls) -> None:
    """
    Adds the full suite of flex arithmetic methods (``pow``, ``mul``, ``add``)
    to the class.

    Parameters
    ----------
    cls : class
        flex methods will be defined and pinned to this class
    """
    ...

