"""
This type stub file was generated by pyright.
"""

"""
Public testing utility functions.
"""
__all__ = ["assert_extension_array_equal", "assert_frame_equal", "assert_series_equal", "assert_index_equal"]
