"""
This type stub file was generated by pyright.
"""

from pandas._config import config, dates
from pandas._config.config import describe_option, get_option, option_context, options, reset_option, set_option
from pandas._config.display import detect_console_encoding

"""
pandas._config is considered explicitly upstream of everything else in pandas,
should have no intra-pandas dependencies.

importing `dates` and `display` ensures that keys needed by _libs
are initialized.
"""
__all__ = ["config", "detect_console_encoding", "get_option", "set_option", "reset_option", "describe_option", "option_context", "options"]
