"""
This type stub file was generated by pyright.
"""

from pandas.compat import is_platform_windows

"""
Hypothesis data generator helpers.
"""
OPTIONAL_INTS = ...
OPTIONAL_FLOATS = ...
OPTIONAL_TEXT = ...
OPTIONAL_DICTS = ...
OPTIONAL_LISTS = ...
OPTIONAL_ONE_OF_ALL = ...
if is_platform_windows():
    DATETIME_NO_TZ = ...
else:
    DATETIME_NO_TZ = ...
DATETIME_JAN_1_1900_OPTIONAL_TZ = ...
DATETIME_IN_PD_TIMESTAMP_RANGE_NO_TZ = ...
INT_NEG_999_TO_POS_999 = ...
YQM_OFFSET = ...
